;File: Walther.asm December 2011
;Development of Walther FP control unit

;Configuration Settings::::::
;Oscillator:  Internal RC No Clock
;Watchdog Timer: Off
;Power Up Timer: On
;Master Clear Enable: Internal
;Code Protect: Off
;Daa EE Read Protect: Off
;Brown Out Detect: BOD and SBOREN disabled
;Internal External Switchover Mode:  Disabled
;Monitor Clock Failsafe: Enabled
;Configuration code: 38C4

;PIC Pinouts:
;Pin	Function	Use
;1	Vdd		
;2	GP5		Output, default high; LED bring low to turn on
;3	GP4		Input, Trigger sensor in
;4	GP3		Input, Loading lever switch 
;5	GP2		Input, PWM out to SMPS FET, becomes output	
;6	GP1		Input, High V feedback
;7	GP0		Output, low, Pulse to fire FET
;8	Ground

;TMR0 used for firing pulse period
;TMR1 used for half second timing loop for flashing LED
;TMR2 used for PWM

	LIST P=12F683
	__config 38C4
	ERRORLEVEL	-302	;stop TRIS warning message
	
TMR0		EQU     H'0001'		;Bank 0	
PCL		EQU     H'0002'
STATUS		EQU     H'0003'
GPIO		EQU     H'0005'
INTCON		EQU     H'000B'
PIR1		EQU     H'000C'
TMR1L		EQU     H'000E'		
TMR1H		EQU     H'000F'		
T1CON		EQU     H'0010'
TMR2		EQU     H'0011'
T2CON		EQU	H'0012'		
CCPR1L		EQU     H'0013'
CCP1CON		EQU     H'0015'
CMCON0		EQU     H'0019'
ADRESH		EQU     H'001E'	
ADCON0		EQU	H'001F'

OPTIONREG	EQU     H'0081'		;Bank 1
TRISIO		EQU     H'0085'
PR2		EQU     H'0092'
VRCON		EQU	H'0099'
ANSEL		EQU	H'009F'

RP0       EQU     H'0005'	;Status Register Bank Select flag
Z         EQU     H'0002'	;Status Register Zero flag
DC        EQU     H'0001'	;Status Register Digit Carry flag
C         EQU     H'0000'	;Status Register Carry flag

Period	EQU	.35			;PWM period
DutyL	EQU	.9			;PWM duty length

VRLive	EQU	0x9d			;VRCON voltage reference value for live fire
VRDry	EQU	0x94			;VRCON voltage reference value for dry fire

Flags		EQU 0x20		;Program flags
DutyVal		EQU 0x21		;Duty value for PWM
PeriodVal	EQU 0x22		;Period value for PWM
Counter1	EQU 0x23		;General purpose counter

#define	TMR2F		PIR1, 1		;Timer2 interrupt flag, used on PWM
#define	TMR1F		PIR1, 0		;Timer1 interrupt flag, used for half second timing loop
#define	TOIF		INTCON, 2	;Timer0 interrupt flag, used for firing pulse period
#define TMR2ON		T2CON, 2	;Timer2 on bit
#define PWMComp		CMCON0, 6	;Comparator flag (COUT) high if capacitor is charged
#define LED		GPIO, 5		;LED Port
#define ADCOn		ADCON0, 1	;Flag for Go/Done bit on ADC
#define LoadLever	GPIO, 3		;Low when breech switch is closed; High when open
#define FireFET		GPIO, 0		;Fire pulse to FET controlling solenoid
#define ChargedFlag	Flags, 0	;1=Charge level reached; 0=charge level not reached
#define PWMFlag		Flags, 1	;1=PWM on, 0=PWM off
#define FlashLEDFlag	Flags, 2	;1=LED is on, 0= LED is off
#define TriggerFlag	Flags, 3	;1= trigger pulled, 0= trigger open

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	org	0x00
Setup	
 	bcf		STATUS,RP0			;GO TO Bank 0
	clrf		GPIO				;Make all pins low		

	bsf		STATUS,RP0			;GO TO Bank 1
	movlw		0x18				;Set FOSC/8 and AN3 for A/D
	movwf		ANSEL				;AD used to read trigger
							
	movlw		b'00011110'
	movwf		TRISIO				;Set I/O per table above
	movlw		0xc5				;GPIO pullups disabled
	movwf		OPTIONREG			;TMR0 on internal clock and 64:1 prescale

	movlw		VRLive				;Live fire default; lever up; bit set
	movwf		VRCON				;Store value to comparator reference

	bcf		STATUS,RP0			;GO TO Bank 0
	movlw		b'00010100'
;			  0				unused
;			   0				Output bit, ready only;
;			    0				unused
;			     1				invert output as Cin+ is reference
;			      0				No effect
;			       100			Cin- analog; Cin+ and Cout digital			

	movwf		CMCON0			;Turn on comparator

	movlw		b'10001101'			;ADC Right Justify output; Vdd Reference
	movwf		ADCON0				;AN3 is ADC, turn on ADC

;Configure PWM and start  based on documentation sequence
	bcf		STATUS,RP0		;GO TO Bank 0
	clrf		CCP1CON
	clrf		TMR2

	bsf		STATUS,RP0		;GO TO Bank 1
	bsf		TRISIO, 2		;Set P1A input for PWM in
	movlw		Period
	movwf		PR2			;Set PWM period 

	bcf		STATUS,RP0		;GO TO Bank 0

	movlw		b'00001100'		;Configure PWM
;			  xx			;P1A output; other PxA pins are I/O
;			    xx			;Leave count 2 low bits 0
;			      xxxx		;Set mode to PWM
	movwf		CCP1CON

	movlw		DutyL
	movwf		CCPR1L			;Set duty cycle

	bcf		TMR2F			;Clear timer2 interrupt flag
	bsf		TMR2ON			;Turn Timer2 on

	btfss		TMR2F			;Wait for interrupt flag to set
	goto		$-1

	bsf		STATUS,RP0		;GO TO Bank 1
	bcf		TRISIO, 2		;Set P1A input for PWM out
	bcf		STATUS,RP0		;GO TO Bank 0

	clrf		CCPR1L			;Default PWM to off
	clrf		Flags			;Default flags to clear

	movlw		0x35
	movwf		T1CON			;Set Timer1 on; prescale 8:1

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;MAIN LOOP  MAIN LOOP  MAIN LOOP  MAIN LOOP  MAIN LOOP  
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

LiveDry				;Select live or dry fire using Load lever on power up
	btfsc		LoadLever			;Test for lever state
	goto		$+4				;Live fire; lever up; bit set
	bsf		STATUS,RP0		;GO TO PAGE 1
	movlw		VRDry				;Dry fire; lever down; bit clear
	movwf		VRCON				;Store value to comparator reference
	bcf		STATUS,RP0		;GO TO PAGE 0

TriggerTest				;Test that trigger is not pulled; wait until opened
	bcf		LED				;Default to LED on
	call		TriggerRead			;Call ADC to read trigger
	btfsc		TriggerFlag			;Test trigger is pulled
	goto		$-2				;Wait until trigger is open
	bsf		LED				;Turn off LED when/if trigger opened

NextShot				;Start of Loop for each shot
	
ShootLoop
	btfss		LoadLever			;Test load lever position
	goto		$+9			;Lever is closed; run charging routine
	nop					;Lever is open; put system on standby
	clrf		CCPR1L				;Turn off PWM
	bcf		PWMFlag				;Clear PWM on flag
	bsf		LED				;Turn off LED 
	call		TriggerRead			;Call ADC to read trigger
	btfsc		TriggerFlag			;Test trigger is pulled; no fire allowed		
	bcf		LED				;Turn on  LED if trigger pulled
	goto		ShootLoop			;Repeat loop waiting for breech to close						;Breech is closed
	call		TriggerRead			;Call ADC to read trigger
	btfsc		TriggerFlag			;Test trigger is pulled	
	goto		TriggerFire			;Trigger is pulled go to firing sequence

HalfSecondChargePeriod				;Breech closed, trigger open, charge circuit
	call		ChargeLoop			;Call process to control PWM
	btfss		TMR1F				;Test flag rollover for 1/2 second
	goto		ShootLoop			;no rollover
	bcf		TMR1F				;Rollover, clear timer flag
	btfsc		ChargedFlag			;If charged flash LED
	call		FlashLED			;Based on flags turn on flashing LED's
	goto		ShootLoop			;Back to start of loop

TriggerFire					;Firing sequence; trigger is read as pulled
	bcf		LED				;Turn on LED noting trigger pulled
	clrf		CCPR1L				;Turn off PWM
	bsf		FireFET				;Power up firing solenoid

	movlw		0x00				;Set timer for pulse period
	movwf		TMR0				;Set up for firing pulse period
	bcf		TOIF				;clear timer rollover flag
	btfss		TOIF				;Loop until timer rolls over
	goto		$-1

	bcf		FireFET				;Stop firing pulse

TriggerRelease	
	bcf		ChargedFlag			;Clear 'gun is charged' flag
	call		TriggerRead			;Call ADC to read trigger
	btfsc		TriggerFlag			;Test trigger is pulled	
	goto		$-2				;Continue loop until trigger is open
	bsf		LED				;Turn off LED as trigger is open

LeverTest
	btfsc		LoadLever			;Breech must be opened before moving on	
	goto		NextShot			;Return to start for next shot
	call		TriggerRead			;Test for trigger pul
	bsf		LED				;Assume trigger is released
	btfsc		TriggerFlag			;Test trigger is pulled	
	bcf		LED				;Light LED as trigger pulled
	goto		LeverTest			;Repeat until load lever is open

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;MAIN LOOP  MAIN LOOP  MAIN LOOP  MAIN LOOP  MAIN LOOP  
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;==================================================================================================
FlashLED			;If LED is lit, turn off; if dark, turn it on  
	btfsc		FlashLEDFlag			;Test the flag
	goto		$+4				;The flag is set

	bcf		LED				;Turn on the LED
	bsf		FlashLEDFlag			;Set the flag
	return

	bsf		LED				;Turn off LED
	bcf		FlashLEDFlag			;Clear the flag
	return	
;==================================================================================================

;Manages charging the gun.  PWMComp is high voltage feedback state from comparator.  

ChargeLoop  
	btfss		PWMComp		;High means capacitor is charged-so turn off
	goto		PWMOn			;Low means cap needs charge--so turn on

PWMOff					;Turn or keep PWM off
	btfss		PWMFlag			;Test flag to see if following has been done
	return					;Don't repeat following steps so return
	clrf		CCPR1L			;Clear duty cycle value of PWM
	bcf		PWMFlag			;Clear flag to show PWM is off
	bsf		ChargedFlag		;PWM turned off because gun is charged
	return

PWMOn					;Turn or keep PWM on
	btfsc		PWMFlag			;Test flag to see if following has been done
	return					;Don't repeat following steps
	movlw		DutyL
	movwf		CCPR1L			;Set duty cycle
	bsf		TMR2ON			;Turn Timer2 on to start PWM

	bsf		PWMFlag			;Set flag showing PWM is active
	return
;==================================================================================================
;ADC is used to read trigger state.  If ADRESH is not 0; trigger is pulled.

TriggerRead   
	bsf	ADCOn			;Turn on ADC conversion request

	btfsc	ADCOn				;Wait for ADC to return digital value
	goto	$-1

	movf	ADRESH, w			;Load AD high byte to working storage
	btfss	STATUS, Z			;Test ADRESH is zero
	goto	$+3

	bcf	TriggerFlag			;Zero flag is set ; trigger open
	return
	
	bsf	TriggerFlag			;Zero flag is clear; trigger pulled
	return
;----------------------------------------------------------------------------

 End
